{% if cls._list_filters %}
**Object listing filters**
{% for item in cls._list_filters %}
- ``{{ item }}``
{% endfor %}
{% endif %}

{% if cls._create_attrs %}
**Object Creation**
{% if cls._create_attrs.required %}
Required attributes for object create:
{% for item in cls._create_attrs.required %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% if cls._create_attrs.optional %}
Optional attributes for object create:
{% for item in cls._create_attrs.optional %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% if cls._create_attrs.exclusive %}
Mutually exclusive attributes for object create:
{% for item in cls._create_attrs.exclusive %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% endif %}

{% if cls._update_attrs %}
**Object update**
{% if cls._update_attrs.required %}
Required attributes for object update:
{% for item in cls._update_attrs.required %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% if cls._update_attrs.optional %}
Optional attributes for object update:
{% for item in cls._update_attrs.optional %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% if cls._update_attrs.exclusive %}
Mutually exclusive attributes for object update:
{% for item in cls._update_attrs.exclusive %}
- ``{{ item }}``
{% endfor %}
{% endif %}
{% endif %}
